@@ -1,5 +1,5 @@
 /* BFD support for the WebAssembly target
-   Copyright (C) 2017-2019 Free Software Foundation, Inc.
+   Copyright (C) 2017-2021 Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
@@ -24,13 +24,13 @@
 #include "libiberty.h"
 
 #define N(number, print, default, next)  \
-{  32, 32, 8, bfd_arch_wasm32, number, "wasm32", "wasm32", 4, default, bfd_default_compatible, \
-   bfd_default_scan, bfd_arch_default_fill, next }
+ { 32, 32, 8, bfd_arch_wasm32, number, "wasm32", "wasm32", 4, default, bfd_default_compatible, \
+   bfd_default_scan, bfd_arch_default_fill, next, 0 }
 
 static const bfd_arch_info_type arch_info_struct[] =
 {
-  N (bfd_mach_wasm32, "wasm32", TRUE, NULL)
+  N (bfd_mach_wasm32, "wasm32", true, NULL)
 };
 
 const bfd_arch_info_type bfd_wasm32_arch =
-  N (bfd_mach_wasm32, "wasm32", TRUE, & arch_info_struct[0]);
+  N (bfd_mach_wasm32, "wasm32", true, & arch_info_struct[0]);
